#include "MUMA.h"

namespace MUMA
{    
     // PROC. Auxiliarers
     string Mayusculas(string s); 
     void CargarListaAsignaturas(ifstream &fich,TLista &l);
     void SalvarListaAsignaturas(ofstream &fich,TLista &l);
     void DuplicaLista(TLista &origen, TLista &destino, TNatural &numElementos);
     void BuscarAsignaturaLista(TLista &l, TNatural codigo, TAsignatura &a, TError &error);

     
     string Centro_a_Cadena(TCentro c)
     {
         string res;
         
         switch(c)
         {
              case informatica: res = "informatica"; break;
              case telecomunicaciones: res = "telecomunicaciones"; break;
              case industriales: res = "industriales"; break;
              case ciencias: res = "ciencias"; break;
         }   
         
         return res;
     }
     
     void Cadena_a_Centro(string s, TCentro &c, bool &ok)
     {
          ok = false;
          c = informatica;
          
          while ( (c<=ciencias) && (!ok) )
          {
                if (Mayusculas(s) == Mayusculas(Centro_a_Cadena(c)))
                {
                    ok = true;              
                }
                else
                {
                    c=SUC(c);
                }
          }
     }
     
     TCentro SUC(TCentro c)
     {
          return TCentro(TNatural(c)+1);   
     }
     
     void CrearUMA(TUMA &tabla)
     {
          TCentro c;
          
          for(c=informatica;c<=ciencias;c=SUC(c))
          {
              tabla[c] = CrearLista();                                   
          }
     }
     
     void DestruirUMA(TUMA &tabla)
     {
          TCentro c;
          
          for(c=informatica;c<=ciencias;c=SUC(c))
          {
              DestruirLista(tabla[c]);
          }
     }
     
     void InsertarAsignaturaUMA(TUMA &tabla, TCentro c, TAsignatura a, TError &error)
     {
          InsertarLista(tabla[c],a,error);
     }
     
     void ListarAsignaturasUMA(TUMA tabla)
     {
          TCentro c;
          
          for(c=informatica;c<=ciencias;c=SUC(c))
          {
              cout << "Centro: " << Centro_a_Cadena(c) << endl;
              MostrarLista(tabla[c]);
              cout << endl; 
          }
     }
     
     void CargarAsignaturasUMAFichero(TUMA &tabla, string nomFich, TError &error)
     {
          ifstream fich;
          string sCentro;
          TCentro c;
          bool ok;
          
          fich.open(nomFich.c_str());
          
          if ( fich.bad() || fich.fail() )
          {
               error = ErrorFichero;
          }
          else
          {
              getline(fich,sCentro,SP);
              while (!fich.eof())
              {
                    Cadena_a_Centro(sCentro, c, ok);
                    CargarListaAsignaturas(fich,tabla[c]);
                    getline(fich,sCentro,SP);
              }
              fich.close();
          }
     }
     
     void SalvarAsignaturasUMAFichero(TUMA &tabla, string nomFich, TError &error)
     {
          ofstream fich;
          TCentro c;
          
          fich.open(nomFich.c_str());
          
          if ( fich.bad() || fich.fail() )
          {
               error = ErrorFichero;
          }
          else
          {                        
              for(c=informatica;c<=ciencias;c=SUC(c))
              {
                  fich << Centro_a_Cadena(c) << SP;
                  SalvarListaAsignaturas(fich,tabla[c]);
              }
              fich.close();
          }
     }
     
     void BuscarAsignaturaUMA(TUMA &tabla, TNatural codigo, 
                              TCentro &c, TAsignatura &a, TError &error)
     {
         
         error = AsignaturaNoExiste;
         c = informatica;
          
         while ( (c<=ciencias) && (error == AsignaturaNoExiste) )
         {
               BuscarAsignaturaLista(tabla[c],codigo,a,error);
               if (error == AsignaturaNoExiste)
               {
                   c = SUC(c);      
               }
         }
     }
     
     void MatricularAsignaturaUMA(TUMA &tabla, TNatural codigo, TError &error)
     {
          TCentro c;
          TAsignatura a;
          
          BuscarAsignaturaUMA(tabla, codigo, c, a, error);
          if (error==NoError)
          {
             EliminarLista(tabla[c], codigo, error);                             
             Matricula(a, 1 , error);
             InsertarLista(tabla[c], a, error);
          }
     }
     
     void GestionAutomaticaMatriculacion(TUMA &tabla, string nomFich)
     {
          ifstream fichEntrada;
          ofstream fichSalida;
          string nomFichSalida;
          string nif;
          TNatural codigo;
          TError error;
          TAsignatura a;
          TCentro c;
          
          nomFichSalida = "Matriculacion." + nomFich;
          fichEntrada.open(nomFich.c_str());
          fichSalida.open(nomFichSalida.c_str());
          
          if (fichEntrada.bad() || fichEntrada.fail() || 
              fichSalida.bad() || fichSalida.fail() )
          {
               error = ErrorFichero;
          }
          else
          {
              getline(fichEntrada,nif,SP);
              while (!fichEntrada.eof())
              {
                  fichEntrada >> codigo;
                  fichEntrada.ignore();
                  BuscarAsignaturaUMA(tabla, codigo, c, a, error);
                  
                  if (error==NoError)
                  {
                     MatricularAsignaturaUMA(tabla, codigo, error);                  
                  }
                  
                  if (error==NoError)
                  {
                     fichSalida << nif << " matriculado en el centro " 
                                << Centro_a_Cadena(c) << " en la asignatura "
                                << codigo << " de la titulacin " 
                                << Titulacion(a) << endl;
                  }
                  getline(fichEntrada,nif,SP);
              }
              fichEntrada.close();
              fichSalida.close();
          }
          
     }
     
     // PROC. Auxiliares
     string Mayusculas(string s) 
     {
            string res;
            TNatural i;
            char c;
            
            res = "";
            
            for(i=0;i<s.length();++i)
            {
                c = toupper(s[i]);
                res+=c;                     
            }
            
            return res;
     }
     
     void CargarListaAsignaturas(ifstream &fich,TLista &l)
     {
          TNatural i,num;
          TAsignatura a;
          TError error;
          
          fich >> num;
          fich.ignore();
          for(i=0;i<num;++i)
          {
             LeeAsignaturaFichero(fich,a);
             InsertarLista(l,a,error);
          }
     }
     
     void SalvarListaAsignaturas(ofstream &fich,TLista &l)
     {
          TNatural i,num;
          TLista copia;
          TAsignatura a;
          TError error;
          
          DuplicaLista(l,copia,num);
          
          fich << num << endl;
          for(i=0;i<num;++i)
          {
             SacarPrimeroLista(copia, a, error);                            
             EscribeAsignaturaFichero(fich,a);
          }
          DestruirLista(copia);
     }

     void DuplicaLista(TLista &origen, TLista &destino, TNatural &numElementos)
     {
          TAsignatura a;
          TError error;
          TLista aux;
          
          aux = CrearLista();
          destino = CrearLista();
          numElementos = 0;
          
          while (!ListaVacia(origen))
          {
              SacarPrimeroLista(origen, a, error);
              InsertarLista(aux, a, error);
              ++numElementos;
          }

          while (!ListaVacia(aux))
          {
              SacarPrimeroLista(aux, a, error);
              InsertarLista(origen, a, error);
              InsertarLista(destino, a, error);
          }

          DestruirLista(aux);
     }
     
     void BuscarAsignaturaLista(TLista &l, TNatural codigo, TAsignatura &a, TError &error)
     {
          TLista copia;
          TNatural num;
          
          DuplicaLista(l,copia,num);
          error = AsignaturaNoExiste;
          
          while ( (!ListaVacia(copia)) && (error==AsignaturaNoExiste) )
          {
             SacarPrimeroLista(copia, a, error);
             if (Codigo(a)==codigo)
             {
                 error = NoError;
             }
             else
             {
                 error = AsignaturaNoExiste;
             }
             
          }
          DestruirLista(copia);
     }     
     
}
